;; Generated package description from mathjax.el  -*- no-byte-compile: t -*-
(define-package "mathjax" "0.0.20241111.94448" "Render formulas using MathJax" 'nil :commit "97290f1ef348a3689fe958eb2d121c8a777d3a8f" :authors '(("Augusto Stoffel" . "arstoffel@gmail.com")) :maintainer '("Augusto Stoffel" . "arstoffel@gmail.com") :keywords '("tex" "text" "tools") :url "https://github.com/astoff/mathjax.el")
